"use strict";
const Sequelize = require("sequelize");
const REGISTRY = new Map();
function register(cls, property, decorator) {
    let map;
    if (REGISTRY.has(cls)) {
        map = REGISTRY.get(cls);
    }
    else {
        map = new Map();
        REGISTRY.set(cls, map);
    }
    let list;
    if (map.has(property)) {
        list = map.get(property);
    }
    else {
        list = [];
        map.set(property, list);
    }
    if (list.indexOf(decorator) < 0) {
        list.push(decorator);
    }
}
function SequelizeTypeHelper(type) {
    return function (cls, property) {
        register(cls.constructor.name, property, type);
    };
}
function STRING() {
    return SequelizeTypeHelper(Sequelize.STRING);
}
exports.STRING = STRING;
function TEXT() {
    return SequelizeTypeHelper(Sequelize.TEXT);
}
exports.TEXT = TEXT;
function INTEGER() {
    return SequelizeTypeHelper(Sequelize.INTEGER);
}
exports.INTEGER = INTEGER;
function FLOAT() {
    return SequelizeTypeHelper(Sequelize.FLOAT);
}
exports.FLOAT = FLOAT;
function REAL() {
    return SequelizeTypeHelper(Sequelize.REAL);
}
exports.REAL = REAL;
function DOUBLE() {
    return SequelizeTypeHelper(Sequelize.DOUBLE);
}
exports.DOUBLE = DOUBLE;
function DECIMAL() {
    return SequelizeTypeHelper(Sequelize.DECIMAL);
}
exports.DECIMAL = DECIMAL;
function DATE() {
    return SequelizeTypeHelper(Sequelize.DATE);
}
exports.DATE = DATE;
function BLOB() {
    return SequelizeTypeHelper(Sequelize.BLOB);
}
exports.BLOB = BLOB;
function BOOLEAN() {
    return SequelizeTypeHelper(Sequelize.STRING);
}
exports.BOOLEAN = BOOLEAN;
class EntityModel {
    constructor(table) {
        this.table = table;
    }
    getListOfAttributeDecorators(property) {
        const name = this.constructor.name;
        return !REGISTRY.has(name) ? [] : REGISTRY.get(name).get(property);
    }
    getfunctions() {
        let funcs = FindFunctions(this);
        funcs = funcs.filter(t => t != "constructor");
        return funcs;
    }
    convertToUnderscopeCase(str) {
        return str
            .replace(/([A-Z])/g, ' $1')
            .replace(/^./, function (str) { return str.toUpperCase(); });
    }
    getColumns() {
        let attr = {};
        for (let key of this.getfunctions()) {
            let type = this.detectTypeOfField(this.getListOfAttributeDecorators(key));
            if (!type) {
                continue;
            }
            attr[key] = {
                field: key,
                type: type
            };
        }
        return attr;
    }
    detectTypeOfField(decorationsList) {
        let type = null;
        if (!decorationsList || !decorationsList.forEach)
            return type;
        decorationsList.forEach(item => {
            if (item == Sequelize.STRING) {
                type = item;
            }
            if (item == Sequelize.INTEGER) {
                type = item;
            }
            if (item == Sequelize.TEXT) {
                type = item;
            }
            if (item == Sequelize.BLOB) {
                type = item;
            }
            if (item == Sequelize.DATE) {
                type = item;
            }
            if (item == Sequelize.DECIMAL) {
                type = item;
            }
            if (item == Sequelize.DOUBLE) {
                type = item;
            }
            if (item == Sequelize.FLOAT) {
                type = item;
            }
        });
        return type;
    }
    static GetInstance(table, connection) {
        let modelInstance = new this(table);
        let columns = modelInstance.getColumns();
        let config = {
            freezeTableName: true // Model tableName will be the same as the model name
        };
        return connection.define(modelInstance.table, columns, config);
    }
}
exports.EntityModel = EntityModel;
function FindFunctions(obj) {
    return Object.getOwnPropertyNames(Object.getPrototypeOf(obj));
}
exports.FindFunctions = FindFunctions;
