const Sequelize = require("sequelize");
const REGISTRY = new Map<string, Map<string, string[]>>();
function register(cls: string, property: string, decorator: string) {
    let map: Map<string, string[]>;

    if (REGISTRY.has(cls)) {
        map = REGISTRY.get(cls);
    } else {
        map = new Map<string, string[]>();
        REGISTRY.set(cls, map);
    }

    let list: string[];
    if (map.has(property)) {
        list = map.get(property);
    } else {
        list = [];
        map.set(property, list);
    }

    if (list.indexOf(decorator) < 0) {
        list.push(decorator);
    }
}

function SequelizeTypeHelper(type: any) {
    return function (cls: any, property: string) {
        register(cls.constructor.name, property, type);
    }
}    
export function STRING() {
    return SequelizeTypeHelper(Sequelize.STRING);
}

export function TEXT() {
    return SequelizeTypeHelper(Sequelize.TEXT);
}

export function INTEGER() {
    return SequelizeTypeHelper(Sequelize.INTEGER);
}

export function FLOAT() {
    return SequelizeTypeHelper(Sequelize.FLOAT);
}

export function REAL() {
    return SequelizeTypeHelper(Sequelize.REAL);
}

export function DOUBLE() {
    return SequelizeTypeHelper(Sequelize.DOUBLE);
}

export function DECIMAL() {
    return SequelizeTypeHelper(Sequelize.DECIMAL);
}

export function DATE() {
    return SequelizeTypeHelper(Sequelize.DATE);
}

export function BLOB() {
    return SequelizeTypeHelper(Sequelize.BLOB);
}

export function BOOLEAN() {
    return SequelizeTypeHelper(Sequelize.STRING);
}
 



export class EntityModel {
    
    constructor(private table: string) {

    }
    public getListOfAttributeDecorators(property: string): string[] {
        const name = this.constructor.name;
        return !REGISTRY.has(name) ? [] : REGISTRY.get(name).get(property);
    }
    
    public getfunctions() {
        let funcs  = FindFunctions(this);
        funcs = funcs.filter(t => t != "constructor");
        return funcs;
    }

    public convertToUnderscopeCase(str) {
        return str
        // insert a space before all caps
        .replace(/([A-Z])/g, ' $1')
        // uppercase the first character
        .replace(/^./, function(str){ return str.toUpperCase(); })
    }
    public getColumns() {

        let attr = {};
        for (let key of this.getfunctions()) {
            let type = this.detectTypeOfField(this.getListOfAttributeDecorators(key));
            if (!type) {
                continue;
            }
            attr[key] = {
                field: key,
                type: type
            }
        } 
        return attr;
    }
    private detectTypeOfField(decorationsList: any[]) {
        let type = null;
        
        if (!decorationsList || !decorationsList.forEach) return type;
        decorationsList.forEach(item => {
           
            if (item == Sequelize.STRING) {
                type = item;
            }
            if (item == Sequelize.INTEGER) {
                type = item;
            }
            if (item == Sequelize.TEXT) {
                type = item;
            }
            if (item == Sequelize.BLOB) {
                type = item;
            }
            if (item == Sequelize.DATE) {
                type = item;
            }
            if (item == Sequelize.DECIMAL) {
                type = item;
            }
            if (item == Sequelize.DOUBLE) {
                type = item;
            }
            if (item == Sequelize.FLOAT) {
                type = item;
            }
             
        });
        return type;
    }


    public static GetInstance(table: string, connection: any) {
        let modelInstance = new this(table);
        let columns = modelInstance.getColumns();
        let config = {
            freezeTableName: true // Model tableName will be the same as the model name
        }
        return connection.define(modelInstance.table, columns, config); 
    }
}

export function FindFunctions(obj: any){
    return Object.getOwnPropertyNames(Object.getPrototypeOf(obj));
}